@@ -301,10 +301,11 @@ emCV = function (y, gen, k=5, n=5, Pi=0.75, alpha=0.02, df=10, R2=0.5, avg=TRUE,
     f5 = emBA(y[-w], gen[-w, ], R2 = R2, df = df)
     f6 = emBB(y[-w], gen[-w, ], Pi = Pi, R2 = R2, df = df)
     f7 = emBC(y[-w], gen[-w, ], Pi = Pi, R2 = R2, df = df)
-    f8 = emML(y[-w], gen[-w, ])
+    f8 = emBD(y[-w], gen[-w, ], R2 = R2, df = df)
+    f9 = emML(y[-w], gen[-w, ])
     cat("DONE WITH CROSS-VALIDATION CYCLE", Seed, "\n")
     NamesMod = c("emRR", "emEN", "emBL", "emDE", "emBA", 
-                 "emBB", "emBC", "emML", "OBSERVATION")
+                 "emBB", "emBC", "emBD", "emML", "OBSERVATION")
     M = matrix(NA, Nk, length(NamesMod))
     colnames(M) = NamesMod
     for (i in 1:(length(NamesMod)-1)) M[, i] = gen[w, ] %*% get(paste("f", i,sep = ""))$b
@@ -327,10 +328,11 @@ emCV = function (y, gen, k=5, n=5, Pi=0.75, alpha=0.02, df=10, R2=0.5, avg=TRUE,
     f5 = emBA(y[-w], gen[-w, ], R2 = R2, df = df)
     f6 = emBB(y[-w], gen[-w, ], Pi = Pi, R2 = R2, df = df)
     f7 = emBC(y[-w], gen[-w, ], Pi = Pi, R2 = R2, df = df)
-    f8 = emML(y[-w], gen[-w, ])
+    f8 = emBD(y[-w], gen[-w, ], R2 = R2, df = df)
+    f9 = emML(y[-w], gen[-w, ])
     cat("DONE WITH CROSS-VALIDATION CYCLE", lev, "\n")
     NamesMod = c("emRR", "emEN", "emBL", "emDE", "emBA", 
-                 "emBB", "emBC", "emML", "OBSERVATION")
+                 "emBB", "emBC", "emBD", "emML", "OBSERVATION")
     M = matrix(NA, Nk, length(NamesMod))
     colnames(M) = NamesMod
     for (i in 1:(length(NamesMod)-1)) M[, i] = gen[w, ] %*% get(paste("f", i,sep = ""))$b
@@ -366,3 +368,4 @@ emCV = function (y, gen, k=5, n=5, Pi=0.75, alpha=0.02, df=10, R2=0.5, avg=TRUE,
   }
   return(sCV(b))
 }
+
