@@ -159,6 +159,65 @@ SEXP emBC(NumericVector y, NumericMatrix gen, double df = 10, double R2 = 0.5, d
                       Named("h2") = h2);
 }
 
+// [[Rcpp::export]]
+SEXP emBD(NumericVector y, NumericMatrix gen, double df = 10, double R2 = 0.5){
+  int it = 200;
+  int p = gen.ncol();
+  int n = gen.nrow();
+  double ve = 1;
+  NumericVector d(p);
+  NumericVector b(p);
+  NumericVector vb = b+1;
+  NumericVector Lmb = ve/vb;
+  double vy = var(y);
+  double Pi = 0.50;
+  NumericVector xx(p);
+  NumericVector vx(p);
+  for(int i=0; i<p; i++){
+    xx[i] = sum(gen(_,i)*gen(_,i));
+    vx[i] = var(gen(_,i));}
+  double MSx = sum(vx)*Pi;
+  double Sb = R2*(df+2)*vy/MSx;
+  double Se = (1-R2)*(df+2)*vy;
+  double mu = mean(y);
+  NumericVector e = y-mu;
+  NumericVector e1(n);
+  NumericVector e2(n);
+  double b0,b1,LR,eM,h2,C;
+  double Pi0 = (1-Pi)/Pi;
+  for(int i=0; i<it; i++){
+    C = -0.5/ve;
+    for(int j=0; j<p; j++){
+      b0 = b[j];
+      b1 = (sum(gen(_,j)*e)+xx[j]*b0)/(xx[j]+Lmb[j]);
+      e1 = e-gen(_,j)*(b1-b0);
+      e2 = e-gen(_,j)*(0-b0);
+      LR = Pi0*exp(C*(sum(e2*e2)-sum(e1*e1)));
+      d[j] = (1/(1+LR));
+      b[j] = b1*d[j];
+      vb[j] = (Sb+b[j]*b[j])/(df+1);
+      e = e - gen(_,j)*(b[j]-b0);}
+    ve = (sum(e*e)+Se)/(n+df);
+    R2 = 1-(sum(e*y)/(n-1))/vy;
+    Pi = mean(d);
+    Sb = R2*(df+2)*vy/MSx/Pi;
+    Se = (1-R2)*(df+2)*vy;
+    Lmb = ve/vb;
+    eM = mean(e);
+    mu = mu+eM;
+    e = e-eM;}
+  h2 = 1-ve/vy;
+  NumericVector fit(n);
+  for(int k=0; k<n; k++){fit[k] = sum(gen(k,_)*b)+mu;}
+  return List::create(Named("mu") = mu,
+                      Named("b") = b,
+                      Named("d") = d,
+                      Named("pi") = 1-Pi,
+                      Named("hat") = fit,
+                      Named("Vb") = vb,
+                      Named("Ve") = ve,
+                      Named("h2") = h2);}
+
 // [[Rcpp::export]]
 SEXP emRR(NumericVector y, NumericMatrix gen, double df = 10, double R2 = 0.5){
   int it = 200;
@@ -268,7 +327,9 @@ SEXP emDE(NumericVector y, NumericMatrix gen, double R2 = 0.5){
   NumericVector e = y-mu;
   // Marker variance
   NumericVector xx(p);
-  for(int k=0; k<p; k++){xx[k] = sum(gen(_,k)*gen(_,k));}
+  for(int k=0; k<p; k++){
+    xx[k] = sum(gen(_,k)*gen(_,k));
+    if(xx[k]==0) xx[k]=0.1;}
   NumericVector vx(p);
   for(int k=0; k<p; k++){vx[k] = var(gen(_,k));}
   double cxx = sum(vx)*(1-R2)/R2;
@@ -991,8 +1052,8 @@ SEXP emGWA(NumericVector y, NumericMatrix gen){
 
 // [[Rcpp::export]]
 SEXP BCpi(NumericVector y, NumericMatrix X,
-              double it = 3000, double bi = 500,
-              double df = 5, double R2 = 0.5){
+          double it = 3000, double bi = 500,
+          double df = 5, double R2 = 0.5){
   // Get dimensions of X
   int p = X.ncol(), n = X.nrow();
   // Estimate crossproducts and MSx
@@ -1002,16 +1063,13 @@ SEXP BCpi(NumericVector y, NumericMatrix X,
     vx[i] = var(X(_,i));}
   double MSx = sum(vx);
   // Get priors
-  double priorA = 1;
-  double priorB = 1;
   double pi = 0.5;
   double vy = var(y);
   double Sb = df*(R2)*vy/MSx/(1-pi);
   double Se = df*(1-R2)*vy;
   double mu = mean(y);
   // Create empty objects
-  double b0,b1,b2,eM,h2,C,MU,VB,VE,Pi,pj,vg,ve=vy,vb=Sb;
-  double PiAlpha,PiBeta,PiMean,PiVar;
+  double b0,b1,b2,eM,h2,C,MU,VB,VE,vee,Pi,pj,ve=vy,vb=Sb;
   NumericVector d(p),b(p),D(p),B(p),fit(n);
   NumericVector e=y-mu,e1(n),e2(n);
   double Lmb=ve/vb;
@@ -1045,11 +1103,8 @@ SEXP BCpi(NumericVector y, NumericMatrix X,
     vb = (sum(b*b)+Sb)/R::rchisq(p+df);
     ve = (sum(e*e)+Se)/R::rchisq(n+df);
     Lmb = ve/vb;
-    // Update Pi from beta
-    PiMean = mean(d); PiVar = var(d);
-    PiAlpha = priorA+((1-PiMean)/PiVar-1/PiMean)*(PiMean*PiMean);
-    PiBeta = priorB+PiAlpha*(1/PiMean-1);
-    pi = R::rbeta(PiAlpha,PiBeta);
+    // Update Pi
+    pi = mean(d);
     Sb = df*(R2)*vy/MSx/(1-pi);
     // Store posterior sums
     if(i>bi){MU=MU+mu; B=B+b; D=D+d; VB=VB+vb; VE=VE+ve; Pi = Pi+pi;}
@@ -1057,11 +1112,12 @@ SEXP BCpi(NumericVector y, NumericMatrix X,
   // Get posterior means
   double MCMC = it-bi;
   MU = MU/MCMC; B = B/MCMC; D = D/MCMC;
-  VB = VB/MCMC; VE = VE/MCMC; Pi = Pi/MCMC;
+  VB = VB/MCMC; VE = VE/MCMC; Pi = 1-Pi/MCMC;
   // Getting GWAS results
   NumericVector PVAL = -log(1-D);
   // Get fitted values and h2
-  vg = VB*MSx/Pi; h2 = vg/(vg+VE);
+  vee = sum(y*e)/(n-1);
+  h2 = 1 - vee/vy;
   for(int k=0; k<n; k++){fit[k] = sum(X(k,_)*B)+MU;}
   // Return output
   return List::create(Named("mu") = MU, Named("b") = B,
@@ -1433,3 +1489,315 @@ SEXP mrr2X(NumericMatrix Y, NumericMatrix X1, NumericMatrix X2){
                       Named("Vb1")=vb1, Named("Vb2")=vb2, 
                       Named("MS1")=MSx1, Named("MS2")=MSx2,
                       Named("Ve")=ve, Named("h2")=h2);}
+
+// [[Rcpp::export]]
+SEXP mkr(NumericMatrix Y, NumericMatrix K){
+  // Convergence parameters
+  int maxit = 200; double tol = 10e-8;
+  // Obtain environment containing function
+  Rcpp::Environment base("package:base");
+  Rcpp::Function solve = base["solve"];
+  Rcpp::Function eigen = base["eigen"];
+  // Eigendecomposition
+  List EIG = eigen(K,true);
+  NumericMatrix X = EIG[1];
+  NumericVector d = EIG[0];
+  // Functions starts here
+  int k = Y.ncol(), p = X.ncol(), n0 = X.nrow();
+  NumericMatrix fit(n0,k),o(n0,k),y(n0,k),e(n0,k);
+  for(int i=0; i<k; i++){
+    o(_,i) = ifelse(is_na(Y(_,i)),0,1);
+    y(_,i) = ifelse(is_na(Y(_,i)),0,Y(_,i));}
+  NumericVector n = colSums(o);
+  // Mu
+  NumericVector mu = colSums(y)/n;
+  for(int j=0; j<k; j++){y(_,j) = (y(_,j)-mu(j))*o(_,j);}
+  // Marker variance
+  NumericMatrix xx(p,k);
+  double MSx = mean(d);
+  for(int i=0; i<p; i++){ for(int j=0; j<k; j++){
+      xx(i,j) = sum(X(_,i)*X(_,i)*o(_,j)); }}
+  // Beta, intersept and residuals
+  NumericMatrix b(p,k),vb(k,k),iG(k,k),rho(k,k),LHS(k,k);
+  NumericVector b0(k),b1(k),vy(k),ve(k),RHS(k);
+  for(int i=0; i<k; i++){for(int j=0; j<k; j++){vb(i,j) = 0;}}
+  for(int i=0; i<k; i++){
+    e(_,i) = y(_,i)+0;
+    vy(i) = sum(e(_,i)*e(_,i))/(n(i)-1);
+    ve(i) = vy(i)*0.5;
+    vb(i,i) = ve(i)/MSx;
+    rho(i,i) = 1;}
+  iG = solve(vb);
+  // Convergence control
+  NumericMatrix bc(p,k);
+  int numit = 0;
+  double cnv = 1;
+  // Loop
+  while(numit<maxit){
+    // Gauss-Seidel loop
+    bc = b+0;
+    for(int j=0; j<p; j++){
+      b0 = b(j,_);
+      LHS = iG+0;
+      for(int i=0; i<k; i++){
+        LHS(i,i) = iG(i,i)+(xx(j,i)/(ve(i)/d(j)) );
+        RHS(i) = (sum(e(_,i)*X(_,j))+xx(j,i)*b0(i))/(ve(i)/d(j));}
+      // Update effects
+      b1 = solve(LHS, RHS);
+      b(j,_) = b1;
+      // Update residuals
+      for(int i=0; i<k; i++){
+        e(_,i) = (e(_,i)-X(_,j)*(b1(i)-b0(i)))*o(_,i);}
+    }
+    // Residual variance components update
+    for(int i=0; i<k; i++){ ve(i) = (sum(e(_,i)*y(_,i)))/n(i);}
+    // Genetic covariance components update
+    for(int i=0; i<n0; i++){ for(int j=0; j<k; j++){fit(i,j) = sum(X(i,_)*b(_,j));}}
+    for(int i=0; i<k; i++){ for(int j=0; j<k; j++){
+      // Diag VC
+      if(i==j){
+        vb(i,j) = (1.01*vy(i)-ve(i))/MSx;
+      }else{
+        vb(i,j) = (sum(fit(_,i)*y(_,j))+sum(fit(_,j)*y(_,i))) / ((n(i)+n(j))*MSx);
+      }}}
+    iG = solve(vb);
+    // Convergence
+    ++numit;
+    cnv = sum(abs(bc-b));
+    if( cnv<tol ){break;}}
+  // Fitting the model
+  NumericVector h2(k); 
+  for(int i=0; i<n0; i++){for(int j=0; j<k; j++){fit(i,j) = sum(X(i,_)*b(_,j))+mu(j);}}
+  h2 = 1-ve/vy;
+  // Genetic correlations
+  NumericMatrix GC(k,k);
+  for(int i=0; i<k; i++){ for(int j=0; j<k; j++){GC(i,j)=vb(i,j)/(sqrt(vb(i,i)*vb(j,j)));}}
+  // Output
+  return List::create(Named("mu")=mu, Named("b")=b,
+                      Named("hat")=fit, Named("h2")=h2,
+                      Named("Vb")=vb, Named("Ve")=ve,
+                      Named("GC")=GC);}
+
+// [[Rcpp::export]]
+SEXP mkr2X(NumericMatrix Y, NumericMatrix K1, NumericMatrix K2){
+  // Convergence parameters
+  int maxit = 200; double tol = 10e-8;
+  // Obtain environment containing function
+  Rcpp::Environment base("package:base");
+  Rcpp::Function solve = base["solve"];
+  Rcpp::Function eigen = base["eigen"];
+  // Eigendecomposition
+  List EIG1 = eigen(K1,true);
+  NumericMatrix X1 = EIG1[1];
+  NumericVector d1 = EIG1[0];
+  List EIG2 = eigen(K2,true);
+  NumericMatrix X2 = EIG2[1];
+  NumericVector d2 = EIG2[0];
+  // Functions starts here
+  int k = Y.ncol(), p1 = X1.ncol(), p2 = X2.ncol(), n0 = X1.nrow();
+  NumericMatrix fit(n0,k),g1(n0,k),g2(n0,k),o(n0,k),y(n0,k),e(n0,k);
+  for(int i=0; i<k; i++){
+    o(_,i) = ifelse(is_na(Y(_,i)),0,1);
+    y(_,i) = ifelse(is_na(Y(_,i)),0,Y(_,i));}
+  NumericVector n = colSums(o);
+  // Mu
+  NumericVector mu = colSums(y)/n;
+  for(int j=0; j<k; j++){y(_,j) = (y(_,j)-mu(j))*o(_,j);}
+  // Marker variance
+  NumericMatrix xx1(p1,k), xx2(p2,k);
+  for(int i=0; i<p1; i++){
+    for(int j=0; j<k; j++){
+      xx1(i,j) = sum(X1(_,i)*X1(_,i)*o(_,j));}}
+  for(int i=0; i<p2; i++){
+    for(int j=0; j<k; j++){
+      xx2(i,j) = sum(X2(_,i)*X2(_,i)*o(_,j));}}
+  double MSx1 = mean(d1), MSx2 = mean(d2);
+  // Beta, intersept and residuals
+  NumericMatrix b1(p1,k),vb1(k,k),iG1(k,k),LHS1(k,k);
+  NumericMatrix b2(p2,k),vb2(k,k),iG2(k,k),LHS2(k,k);
+  NumericVector b_0(k),b_1(k),vy(k),ve(k),RHS1(k),RHS2(k);
+  for(int i=0; i<k; i++){
+    for(int j=0; j<k; j++){
+      vb2(i,j) = 0; vb1(i,j) = 0;}}
+  for(int i=0; i<k; i++){
+    e(_,i) = y(_,i)+0;
+    vy(i) = sum(e(_,i)*e(_,i))/(n(i)-1);
+    ve(i) = vy(i)*0.5;
+    vb1(i,i) = ve(i)/MSx1;
+    vb2(i,i) = ve(i)/MSx2;
+  }
+  iG1 = solve(vb1);
+  iG2 = solve(vb2);
+  // Convergence control
+  NumericMatrix bc1(p1,k), bc2(p2,k);
+  int numit = 0;
+  double cnv = 1;
+  // Loop
+  while(numit<maxit){
+    // Convergence parameter (coef at time zero)
+    bc1 = b1+0; bc2 = b2+0;
+    // Gauss-Seidel loop 1
+    for(int j=0; j<p1; j++){
+      b_0 = b1(j,_);
+      LHS1 = iG1+0;
+      for(int i=0; i<k; i++){
+        LHS1(i,i) = iG1(i,i)+(xx1(j,i)/(ve(i)/d1(j)));
+        RHS1(i) = (sum(e(_,i)*X1(_,j))+xx1(j,i)*b_0(i))/(ve(i)/d1(j));}
+      b_1 = solve(LHS1, RHS1);
+      b1(j,_) = b_1;
+      for(int i=0; i<k; i++){
+        e(_,i) = (e(_,i)-X1(_,j)*(b_1(i)-b_0(i)))*o(_,i);}
+    }
+    // Gauss-Seidel loop 2
+    for(int j=0; j<p2; j++){
+      b_0 = b2(j,_);
+      LHS2 = iG2+0;
+      for(int i=0; i<k; i++){
+        LHS2(i,i) = iG2(i,i)+(xx2(j,i)/(ve(i)/d2(j)));
+        RHS2(i) = (sum(e(_,i)*X2(_,j))+xx2(j,i)*b_0(i))/(ve(i)/d2(j));}
+      b_1 = solve(LHS2, RHS2);
+      b2(j,_) = b_1;
+      for(int i=0; i<k; i++){
+        e(_,i) = (e(_,i)-X2(_,j)*(b_1(i)-b_0(i)))*o(_,i);}
+    }
+    // Residual variance components update
+    for(int i=0; i<k; i++){ ve(i) = (sum(e(_,i)*y(_,i)))/n(i);}
+    // Genetic covariance components update
+    for(int i=0; i<n0; i++){
+      for(int j=0; j<k; j++){
+        g1(i,j) = sum(X1(i,_)*b1(_,j));
+        g2(i,j) = sum(X2(i,_)*b2(_,j));  
+      }}
+    for(int i=0; i<k; i++){
+      for(int j=0; j<k; j++){
+        vb1(i,j) = (sum(g1(_,i)*y(_,j))+sum(g1(_,j)*y(_,i))) / ((n(i)+n(j))*MSx1);
+        vb2(i,j) = (sum(g2(_,i)*y(_,j))+sum(g2(_,j)*y(_,i))) / ((n(i)+n(j))*MSx2);
+      }}
+    iG1 = solve(vb1);
+    iG2 = solve(vb2);
+    // Convergence
+    ++numit;
+    cnv = sum(abs(bc1-b1))+sum(abs(bc2-b2));
+    if( cnv<tol ){break;}}
+  // Fitting the model
+  NumericVector h2(k); 
+  for(int j=0; j<k; j++){ fit(_,j) = mu(j)+g1(_,j)+g2(_,j); }
+  h2 = 1-ve/vy;
+  // Genetic correlations
+  NumericMatrix GC1(k,k),GC2(k,k);
+  for(int i=0; i<k; i++){ for(int j=0; j<k; j++){
+    GC1(i,j)=vb1(i,j)/(sqrt(vb1(i,i)*vb1(j,j)));
+    GC2(i,j)=vb2(i,j)/(sqrt(vb2(i,i)*vb2(j,j)));
+    }}
+  // Output
+  return List::create(Named("mu")=mu,
+                      Named("b1")=b1, Named("b2")=b2,
+                      Named("hat")=fit,
+                      Named("g1")=g1, Named("g2")=g2, 
+                      Named("Vb1")=vb1, Named("Vb2")=vb2,
+                      Named("GC1")=GC1, Named("GC2")=GC2,
+                      Named("Ve")=ve, Named("h2")=h2);}
+
+// [[Rcpp::export]]
+SEXP mrrFast(NumericMatrix Y, NumericMatrix X){
+  int maxit = 200; double tol = 10e-8;  
+  // Obtain environment containing function
+  Rcpp::Environment base("package:base");
+  Rcpp::Function solve = base["solve"];  
+  // Functions starts here
+  int k = Y.ncol(), p = X.ncol(), n0 = X.nrow();
+  NumericMatrix fit(n0,k),o(n0,k),y(n0,k),e(n0,k);
+  for(int i=0; i<k; i++){
+    o(_,i) = ifelse(is_na(Y(_,i)),0,1);
+    y(_,i) = ifelse(is_na(Y(_,i)),0,Y(_,i));  }
+  NumericVector n = colSums(o);  
+  // Mu
+  NumericVector mu = colSums(y)/n;
+  for(int j=0; j<k; j++){
+    y(_,j) = (y(_,j)-mu(j))*o(_,j);  }  
+  // Marker variance
+  NumericMatrix xx(p,k), vx(p,k);
+  double tmp;
+  for(int i=0; i<p; i++){
+    for(int j=0; j<k; j++){
+      xx(i,j) = sum(X(_,i)*X(_,i)*o(_,j));
+      tmp = sum(X(_,i)*o(_,j))/n(j);
+      vx(i,j) = xx(i,j)/n(j)-tmp*tmp;}}
+  NumericVector MSx = colSums(vx);  
+  // Beta, intersept and residuals
+  NumericMatrix b(p,k),vb(k,k),rho(k,k);
+  NumericVector b0(k),vy(k),ve(k);
+  double b1,lhs,rhs,covs,vb0,lambda;
+  for(int i=0; i<k; i++){for(int j=0; j<k; j++){vb(i,j) = 0;}}
+  for(int i=0; i<k; i++){
+    e(_,i) = y(_,i)+0;
+    vy(i) = sum(e(_,i)*e(_,i))/(n(i)-1);
+    ve(i) = vy(i)*0.5;
+    vb(i,i) = ve(i)/MSx(i);
+    rho(i,i) = 1;}
+  NumericMatrix iG = solve(vb);  
+  // Convergence control
+  NumericMatrix bc(p,k);
+  int numit = 0;
+  double cnv = 1;  
+  // Loop
+  while(numit<maxit){    
+    // Gauss-Seidel loop
+    bc = b+0;
+    for(int j=0; j<p; j++){      
+      // Current marker effect
+      b0 = b(j,_);      
+      // Update coefficient one trait at a time
+      for(int i=0; i<k; i++){        
+        // Compute coefficient covariances among traits
+        covs = 0.0;
+        for(int ii=0; ii<k; ii++){
+          if(i!=ii){ covs = covs+iG(i,ii)*b0(ii); } }
+        //covs = covs/ve(i);
+        covs = covs*ve(i);
+        lambda = iG(i,i)*ve(i);        
+        // Set LHS RHS
+        lhs = xx(j,i) + lambda;
+        rhs = sum(e(_,i)*X(_,j)) + xx(j,i)*b0(i) - covs;        
+        // Update effects
+        b1 = rhs/lhs;
+        b(j,i) = b1;        
+        // Update residuals
+        //e(_,i) = ifelse( o(_,i)==1, (e(_,i)-X(_,j)*(b1-b0(i))), 0 );
+        e(_,i) = (e(_,i)-X(_,j)*(b1-b0(i)))*o(_,i); }}    
+    // Update variance components every few iterations
+    if(numit%3==0){      
+      // Get fitted values
+      for(int i=0; i<n0; i++){
+        for(int j=0; j<k; j++){
+          fit(i,j) = sum(X(i,_)*b(_,j));}}      
+      // Residual variance components update
+      for(int i=0; i<k; i++){ ve(i) = (sum(e(_,i)*y(_,i)))/(n(i)-1); }      
+      // Genetic covariance components update
+      for(int i=0; i<k; i++){
+        for(int j=0; j<k; j++){
+          if(i==j){
+                //vb(i,i) = sum(fit(_,i)*y(_,i))/(n(i)*MSx(i));
+                vb(i,i) = (1.1*vy(i)-ve(i))/MSx(i);
+              }else{
+            if(i>j){
+                vb0 = (sum(fit(_,i)*y(_,j))+sum(fit(_,j)*y(_,i))) / ((n(i)*MSx(i))+(n(j)*MSx(j)));
+                vb(i,j) = vb0; vb(j,i) = vb0;}}}}      
+      // Generalized inverse of G2A_Kernels
+      iG = solve(vb);}    
+    // Convergence
+    ++numit;
+    cnv = sum(abs(bc-b));
+    if( cnv<tol ){break;}}  
+  // Fitting the model
+  NumericVector h2(k); 
+  for(int i=0; i<n0; i++){for(int j=0; j<k; j++){fit(i,j) = sum(X(i,_)*b(_,j))+mu(j);}}
+  h2 = 1-ve/vy;  
+  // Genetic correlations
+  NumericMatrix GC(k,k);
+  for(int i=0; i<k; i++){ for(int j=0; j<k; j++){GC(i,j)=vb(i,j)/(sqrt(vb(i,i)*vb(j,j)));}}  
+  // Output
+  return List::create(Named("mu")=mu, Named("b")=b,
+                      Named("hat")=fit, Named("h2")=h2,
+                      Named("Vb")=vb, Named("GC")=GC,
+                      Named("Ve")=ve);}
